[Unit]
Description=%i uWSGI service

[Service]
ExecStart={{ uwsgi_install_prefix.stdout | default('/usr/bin') }}/uwsgi \
          --ini /etc/uwsgi/apps-enabled/%i.ini
SyslogIdentifier=%i
# Has to be root, otherwise uWSGI won't be able to drop privileges
User=root
Restart=on-failure
KillSignal=SIGQUIT
Type=notify
NotifyAccess=all

[Install]
WantedBy=multi-user.target
