<domain type='{{ test_vm_domain_type }}'>
  <name>{{ vm_name }}</name>
  <memory unit='MiB'>{{ test_vm_memory_size }}</memory>
  <vcpu>{{ test_vm_cpu_count }}</vcpu>
  <os>
    <type arch='{{ test_vm_arch }}' machine='{{ test_vm_machine }}'>hvm</type>
    {% if default_boot_mode == 'uefi' %}
      <loader readonly='yes' type='pflash'>{{ efi_loader_path }}</loader>
      <nvram template='{{ efi_nvram_path }}'>/var/lib/libvirt/nvram-{{ vm_name }}.fd</nvram>
    {% else %}
      <boot dev='network'/>
    {% endif %}
    <bootmenu enable='no'/>
    <bios useserial='yes' rebootTimeout='10000'/>
  </os>
  {% if test_vm_domain_type == 'kvm' %}
  <cpu mode='host-passthrough'/>
  {% else %}
  <cpu mode='host-model'/>
  {% endif %}
  <features>
    <acpi/>
    <apic/>
    <pae/>
  </features>
  <clock offset='utc'/>
  <on_poweroff>destroy</on_poweroff>
  <on_reboot>restart</on_reboot>
  <on_crash>restart</on_crash>
  <devices>
    <emulator>{{ test_vm_emulator }}</emulator>
    <disk type='file' device='disk'>
      <driver name='qemu' type='qcow2' cache='{{ test_vm_disk_cache }}'/>
      <source file='{{ vm_volume_path }}'/>
      <target dev='vda' bus='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>
    </disk>
    <controller type='sata' index='0'/>
    <interface type='network'>
      <source network='{{ test_vm_network }}'/>
      <model type='{{ test_vm_nic }}'/>
      {% if default_boot_mode == 'uefi' %}
        <boot order='1'/>
      {% endif %}
    </interface>
    <input type='mouse' bus='ps2'/>
    <serial type='file'>
      <source path='{{ vm_log_file }}'/>
      <target port='0'/>
      <alias name='serial0'/>
    </serial>
    <serial type='pty'>
      <source path='/dev/pts/49'/>
      <target port='1'/>
      <alias name='serial1'/>
    </serial>
    <console type='file'>
      <source path='{{ vm_log_file }}'/>
      <target type='serial' port='0'/>
      <alias name='serial0'/>
    </console>
    <memballoon model='virtio'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x07' function='0x0'/>
    </memballoon>
  </devices>
</domain>
