SUSHY_EMULATOR_BOOT_LOADER_MAP = {
{% if efi_loader_path is defined %}
    'UEFI': {
        'x86_64': '{{ efi_loader_path }}'
    },
{% endif %}
    'Legacy': {
        'x86_64': None
    }
}

SUSHY_EMULATOR_AUTH_FILE = '/etc/redfish-emulator.htpasswd'
SUSHY_EMULATOR_LISTEN_IP = '{{ redfish_emulator_host }}'
SUSHY_EMULATOR_LISTEN_PORT = {{ redfish_emulator_port }}
SUSHY_EMULATOR_STATE_DIR = '{{ redfish_emulator_state_dir }}'
# Sadly this is how real hardware works:
SUSHY_EMULATOR_VMEDIA_VERIFY_SSL = False
